//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// oam.cpp
//
// oam emulation
//
// $Id: $
//

#include <stdlib.h>
#include "oam.h"
#include "nes6502.h"
#include "assert.h"


oam::oam()
{
   reset(true);
}


static inline uint8 miniRand(uint8 value)
{
   value = (value << 1) | (value >> 7);
   value ^= 0xff;
   return value;
}

// oam gets trashed on power up
void oam::reset(bool hardReset)
{
   resetCache();
   
   if (hardReset)
   {
      // just trash whatever is in OAM
      for (int i = 0; i < OAM_SIZE; i++)
      {
         mMemory.memory[i] = miniRand(mMemory.memory[i]);
      }
   }
}


void oam::dma(uint8 address, uint8 value)
{
   // a cpu write to the ppu registers triggers oam dma.
   // since oam dma steals cycles away from the cpu, we must
   // release the cpu's current timeslice.
   nes6502_release();
   nes6502_burn(OAM_DMA_CYCLES);

   uint32 cpuAddress = (uint32) value * 256;
   uint8 addr = address;

   // oam dma starts at current oam address
   do
   {
      mMemory.memory[addr++] = nes6502_getbyte(cpuAddress++);
   }
   while (addr != address);

   // TODO: investigate weird priority swapping here...
}


int oam::cache(oamtemp_t *pTemp)
{
   ASSERT(mNumCached < MAX_CACHED);

   if (mNumCached < MAX_CACHED)
   {
      mCache[mNumCached++] = *pTemp;
      if (mNumCached < MAX_CACHED)
         return 0;
   }

   return -1;
}
