//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// oam.h
//
// oam emulation
//
// $Id: $
//

#ifndef _OAM_H_
#define _OAM_H_

#include "typedefs.h"
#include "assert.h"

#define  MAX_OBJ        64

// oam attribute byte bitmasks
#define  OAMF_VFLIP     0x80
#define  OAMF_HFLIP     0x40
#define  OAMF_BEHIND    0x20

// we use this flag for evil, not good.
#define  OAMF_SPRITE_0  0x10

struct obj_t
{
   uint8    y;
   uint8    tile;
   uint8    attrib;
   uint8    x;
};

struct oamtemp_t
{
   uint8    attrib;  // 3 bits from hardware, plus sprite 0 flag
   uint8    x;
   uint8    pat0;
   uint8    pat1;
};

class oam
{
public:
   oam();

   void  reset(bool hardReset);

   void  dma(uint8 oam_addr, uint8 value);
   void  write(uint8 oam_addr, uint8 value);
   const obj_t *getOBJ(int index) const;

   void  resetCache();
   int   getNumCached() const;
   const oamtemp_t *getCached(int num) const;
   int   cache(oamtemp_t *pTemp);

private:
   enum
   {
      OAM_SIZE = 256,
      OAM_DMA_CYCLES = 512, // cpu cycles per dma
      MAX_CACHED = 8,
   };

   union oam_t
   {
      uint8 memory[OAM_SIZE];
      obj_t obj[MAX_OBJ];
   };

   oam_t       mMemory;
   oamtemp_t   mCache[MAX_CACHED];
   int         mNumCached;
};

inline void oam::write(uint8 address, uint8 value)
{
   mMemory.memory[address] = value;
}

inline const obj_t *oam::getOBJ(int index) const
{
   ASSERT(index < MAX_OBJ);
   return &mMemory.obj[index];
}

inline void oam::resetCache()
{
   mNumCached = 0;
}

inline int oam::getNumCached() const
{
   return mNumCached;
}

inline const oamtemp_t *oam::getCached(int num) const
{
   ASSERT(num < MAX_CACHED);
   return &mCache[num];
}

#endif // !_OAM_H_
