//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// palette.cpp
//
// palette generation routines
//
// $Id: $
//

#include <math.h>
#include "palette.h"

#ifndef M_PI
#define M_PI 3.1415926535897932384626433832795
#endif


static const float brightness[4][4] = 
{
   { 0.50f, 0.75f, 1.00f, 1.00f },
   { 0.29f, 0.45f, 0.73f, 0.90f },
   { 0.00f, 0.24f, 0.47f, 0.77f },
   { 0.02f, 0.04f, 0.05f, 0.07f }
};

static const int colorAngles[16] =
{
   0, 240, 210, 180, 150, 120, 90, 60, 30, 0, 330, 300, 270, 0, 0, 0
};


palette::palette()
{
   mHue = defaultHue;
   mTint = defaultTint;
}


void  palette::generate(color pPalette[NUM_PALETTE_COLORS])
{
   int x, z;
   int r, g, b;
   float s, y, theta;

   for (x = 0; x < 4; x++)
   {
      for (z = 0; z < 16; z++)
      {
         switch (z)
         {
         case 0:
            // color $x0, get luma
            s = 0;
            y = brightness[0][x];
            break;

         case 13:
            // color $xD, get luma
            s = 0;
            y = brightness[2][x];
            break;

         case 14:
         case 15:
            // color $xE/F, set to black
            s = 0;
            y = brightness[3][x];
            
            break;

         default:
            // get tint and default luminance
            s = mTint;
            y = brightness[1][x];
            break;
         }

         theta = (float) (M_PI * ((colorAngles[z] + mHue) / 180.0));

         r = (int) (256.0 * (y + s * sin(theta)));
         g = (int) (256.0 * (y - (27 * s * sin(theta) / 53.0) + (10 * s * cos(theta) / 53.0)));
         b = (int) (256.0 * (y - s * cos(theta)));

         if (r > 255)
            r = 255;
         else if (r < 0)
            r = 0;

         if (g > 255)
            g = 255;
         else if (g < 0)
            g = 0;

         if (b > 255)
            b = 255;
         else if (b < 0)
            b = 0;

         pPalette[x * 16 + z].r = r;
         pPalette[x * 16 + z].g = g;
         pPalette[x * 16 + z].b = b;
      }
   }   
}
